@@ -140,20 +140,6 @@ anjuta_property_data_type_get_type (void)
   }
   return etype;
 }
-GType
-anjuta_preferences_filter_type_get_type (void)
-{
-  static GType etype = 0;
-  if (etype == 0) {
-    static const GEnumValue values[] = {
-      { ANJUTA_PREFERENCES_FILTER_NONE, "ANJUTA_PREFERENCES_FILTER_NONE", "none" },
-      { ANJUTA_PREFERENCES_FILTER_PROJECT, "ANJUTA_PREFERENCES_FILTER_PROJECT", "project" },
-      { 0, NULL, NULL }
-    };
-    etype = g_enum_register_static (g_intern_static_string ("AnjutaPreferencesFilterType"), values);
-  }
-  return etype;
-}
 
 /* enumerations from "anjuta-launcher.h" */
 GType
@@ -230,6 +216,69 @@ anjuta_vcs_status_get_type (void)
   return etype;
 }
 
+/* enumerations from "gbf-project.h" */
+GType
+if_if_get_type (void)
+{
+  static GType etype = 0;
+  if (etype == 0) {
+    static const GEnumValue values[] = {
+      { 0, NULL, NULL }
+    };
+    etype = g_enum_register_static (g_intern_static_string ("if"), values);
+  }
+  return etype;
+}
+
+/* enumerations from "anjuta-project.h" */
+GType
+anjuta_project_node_type_get_type (void)
+{
+  static GType etype = 0;
+  if (etype == 0) {
+    static const GEnumValue values[] = {
+      { ANJUTA_PROJECT_UNKNOWN, "ANJUTA_PROJECT_UNKNOWN", "unknown" },
+      { ANJUTA_PROJECT_GROUP, "ANJUTA_PROJECT_GROUP", "group" },
+      { ANJUTA_PROJECT_TARGET, "ANJUTA_PROJECT_TARGET", "target" },
+      { ANJUTA_PROJECT_SOURCE, "ANJUTA_PROJECT_SOURCE", "source" },
+      { ANJUTA_PROJECT_VARIABLE, "ANJUTA_PROJECT_VARIABLE", "variable" },
+      { 0, NULL, NULL }
+    };
+    etype = g_enum_register_static (g_intern_static_string ("AnjutaProjectNodeType"), values);
+  }
+  return etype;
+}
+GType
+anjuta_project_target_class_get_type (void)
+{
+  static GType etype = 0;
+  if (etype == 0) {
+    static const GEnumValue values[] = {
+      { ANJUTA_TARGET_UNKNOWN, "ANJUTA_TARGET_UNKNOWN", "unknown" },
+      { ANJUTA_TARGET_SHAREDLIB, "ANJUTA_TARGET_SHAREDLIB", "sharedlib" },
+      { ANJUTA_TARGET_STATICLIB, "ANJUTA_TARGET_STATICLIB", "staticlib" },
+      { ANJUTA_TARGET_EXECUTABLE, "ANJUTA_TARGET_EXECUTABLE", "executable" },
+      { ANJUTA_TARGET_PYTHON, "ANJUTA_TARGET_PYTHON", "python" },
+      { ANJUTA_TARGET_JAVA, "ANJUTA_TARGET_JAVA", "java" },
+      { ANJUTA_TARGET_LISP, "ANJUTA_TARGET_LISP", "lisp" },
+      { ANJUTA_TARGET_HEADER, "ANJUTA_TARGET_HEADER", "header" },
+      { ANJUTA_TARGET_MAN, "ANJUTA_TARGET_MAN", "man" },
+      { ANJUTA_TARGET_INFO, "ANJUTA_TARGET_INFO", "info" },
+      { ANJUTA_TARGET_GENERIC, "ANJUTA_TARGET_GENERIC", "generic" },
+      { ANJUTA_TARGET_DATA, "ANJUTA_TARGET_DATA", "data" },
+      { ANJUTA_TARGET_EXTRA, "ANJUTA_TARGET_EXTRA", "extra" },
+      { ANJUTA_TARGET_INTLTOOL, "ANJUTA_TARGET_INTLTOOL", "intltool" },
+      { ANJUTA_TARGET_CONFIGURE, "ANJUTA_TARGET_CONFIGURE", "configure" },
+      { ANJUTA_TARGET_IDL, "ANJUTA_TARGET_IDL", "idl" },
+      { ANJUTA_TARGET_MKENUMS, "ANJUTA_TARGET_MKENUMS", "mkenums" },
+      { ANJUTA_TARGET_GENMARSHAL, "ANJUTA_TARGET_GENMARSHAL", "genmarshal" },
+      { 0, NULL, NULL }
+    };
+    etype = g_enum_register_static (g_intern_static_string ("AnjutaProjectTargetClass"), values);
+  }
+  return etype;
+}
+
 #define __ANJUTA_ENUM_TYPES_C__
 
 /* Generated data ends here */
