@@ -51,7 +51,10 @@ struct _AnjutaBookmarksPrivate
 	GtkTreeViewColumn* column;
 	
 	GtkWidget* button_add;
-	GtkWidget* button_remove;	
+	GtkWidget* button_remove;
+	GtkWidget* grip;
+
+	GtkWidget* menu;
 	
 	DocmanPlugin* docman;
 };
@@ -94,6 +97,38 @@ on_remove_clicked (GtkWidget* button, AnjutaBookmarks* bookmarks)
 	anjuta_bookmarks_remove (bookmarks);
 }
 
+static void
+on_rename (GtkWidget* menuitem, AnjutaBookmarks* bookmarks)
+{
+	AnjutaBookmarksPrivate* priv = BOOKMARKS_GET_PRIVATE(bookmarks);
+	GtkTreeSelection* selection = 
+			gtk_tree_view_get_selection (GTK_TREE_VIEW (priv->tree));
+
+	GList* rows =
+		gtk_tree_selection_get_selected_rows (selection, NULL);
+
+	GtkTreePath* path;
+
+	// We shouldn't get here otherwise
+	g_assert (g_list_length(rows) == 1);
+	
+	g_object_set (G_OBJECT(priv->renderer), "editable", TRUE, NULL);
+
+	path = rows->data; // First and only row
+	anjuta_shell_present_widget (ANJUTA_PLUGIN(priv->docman)->shell,
+	                             priv->window, NULL);
+	gtk_widget_grab_focus (priv->tree);
+	gtk_tree_view_scroll_to_cell (GTK_TREE_VIEW (priv->tree), path,
+	                              priv->column, FALSE, 0.0, 0.0);
+
+	gtk_tree_view_set_cursor_on_cell (GTK_TREE_VIEW (priv->tree), path,
+	                                  priv->column,
+	                                  priv->renderer,
+	                                  TRUE);
+	g_list_foreach (rows, (GFunc)gtk_tree_path_free, NULL);
+	g_list_free (rows);
+}
+
 static void
 on_row_activate (GtkTreeView* view, GtkTreePath* path,
 				 GtkTreeViewColumn* column, AnjutaBookmarks* bookmarks)
@@ -112,6 +147,43 @@ on_row_activate (GtkTreeView* view, GtkTreePath* path,
 	g_object_unref (file);
 }
 
+static gboolean
+on_button_press_event (GtkWidget *widget, GdkEventButton *event, AnjutaBookmarks* bookmarks)
+{
+	AnjutaBookmarksPrivate* priv = BOOKMARKS_GET_PRIVATE(bookmarks);
+	/* Ignore double-clicks and triple-clicks */
+	if (event->button == 3 && event->type == GDK_BUTTON_PRESS)
+	{
+		gint button;
+		gint event_time;
+		
+		GtkTreeSelection* selection = 
+			gtk_tree_view_get_selection (GTK_TREE_VIEW (widget));
+
+		if (gtk_tree_selection_count_selected_rows (selection) == 1)
+		{
+			/*g_signal_connect (menu, "deactivate", 
+			                  G_CALLBACK (gtk_widget_destroy), NULL);*/
+			
+			if (event)
+			{
+				button = event->button;
+				event_time = event->time;
+			}
+			else
+			{
+				button = 0;
+				event_time = gtk_get_current_event_time ();
+			}
+			gtk_menu_popup (GTK_MENU(priv->menu), NULL, NULL, NULL, NULL, button, event_time);
+
+			return TRUE;
+		}
+	}
+
+	return FALSE;
+}
+
 static void
 on_document_added (AnjutaDocman* docman, IAnjutaDocument* doc,
 				   AnjutaBookmarks* bookmarks)
@@ -184,9 +256,9 @@ on_title_edited (GtkCellRendererText *cell,
 	
 	gtk_tree_model_get_iter (model, &iter, path);
 	gtk_list_store_set (GTK_LIST_STORE (model), &iter, COLUMN_TEXT, new_text, -1);
-	
-	gtk_tree_path_free (path);
+
 	g_object_set (G_OBJECT(priv->renderer), "editable", FALSE, NULL);
+	gtk_tree_path_free (path);
 }
 
 static void
@@ -194,8 +266,10 @@ anjuta_bookmarks_init (AnjutaBookmarks *bookmarks)
 {
 	AnjutaBookmarksPrivate* priv = BOOKMARKS_GET_PRIVATE(bookmarks);
 	GtkWidget* scrolled_window;
-	GtkWidget* button_box;
 	GtkTreeSelection* selection;
+
+	GtkWidget* item_rename;
+	GtkWidget* item_remove;
 	
 	priv->window = gtk_vbox_new (FALSE, 5);
 	scrolled_window = gtk_scrolled_window_new (NULL, NULL);
@@ -223,25 +297,57 @@ anjuta_bookmarks_init (AnjutaBookmarks *bookmarks)
 	
 	g_signal_connect (G_OBJECT(priv->tree), "row-activated", G_CALLBACK(on_row_activate),
 					  bookmarks);
+	g_signal_connect (G_OBJECT(priv->tree), "button-press-event", G_CALLBACK(on_button_press_event),
+					  bookmarks);
 	
 	selection = gtk_tree_view_get_selection (GTK_TREE_VIEW(priv->tree));
 	gtk_tree_selection_set_mode (selection, GTK_SELECTION_MULTIPLE);
 	g_signal_connect (G_OBJECT(selection), "changed", G_CALLBACK(on_selection_changed),
 					  bookmarks);
 	
-	button_box = gtk_hbutton_box_new ();
-	priv->button_add = gtk_button_new_from_stock (GTK_STOCK_ADD);
-	priv->button_remove = gtk_button_new_from_stock (GTK_STOCK_REMOVE);
+	priv->grip = gtk_hbox_new (FALSE, 0);
+	priv->button_add = gtk_button_new();
+	gtk_widget_set_tooltip_text (priv->button_add,
+	                             _("Add bookmark"));
+	gtk_container_add (GTK_CONTAINER (priv->button_add),
+	                   gtk_image_new_from_stock (GTK_STOCK_ADD,
+	                                             GTK_ICON_SIZE_MENU));
+	priv->button_remove = gtk_button_new();
+	gtk_widget_set_tooltip_text (priv->button_remove,
+	                             _("Remove bookmark"));
+	gtk_container_add (GTK_CONTAINER (priv->button_remove),
+	                   gtk_image_new_from_stock (GTK_STOCK_REMOVE,
+	                                             GTK_ICON_SIZE_MENU));
 	g_signal_connect (G_OBJECT(priv->button_add), "clicked", G_CALLBACK(on_add_clicked), bookmarks);
 	g_signal_connect (G_OBJECT(priv->button_remove), "clicked", G_CALLBACK(on_remove_clicked), bookmarks);
 	gtk_widget_set_sensitive (GTK_WIDGET(priv->button_add), FALSE);
 	gtk_widget_set_sensitive (GTK_WIDGET(priv->button_remove), FALSE);
-	gtk_box_pack_start (GTK_BOX(button_box), priv->button_add, TRUE, TRUE, 0);
-	gtk_box_pack_start (GTK_BOX(button_box), priv->button_remove, TRUE, TRUE, 0);
 
-	gtk_box_pack_start(GTK_BOX(priv->window), 
-					   button_box,
-					   FALSE, FALSE, 0);
+	gtk_box_pack_start (GTK_BOX(priv->grip),
+	                    gtk_image_new_from_stock (ANJUTA_STOCK_BOOKMARK_TOGGLE,
+	                                              GTK_ICON_SIZE_MENU),
+	                    FALSE, FALSE, 1);
+	gtk_box_pack_start (GTK_BOX(priv->grip),
+	                    gtk_label_new (_("Bookmarks")),
+	                    FALSE, FALSE, 1);
+	gtk_box_pack_start (GTK_BOX(priv->grip), priv->button_add, FALSE, FALSE, 0);
+	gtk_box_pack_start (GTK_BOX(priv->grip), priv->button_remove, FALSE, FALSE, 0);
+	gtk_widget_show_all (priv->grip);
+
+	/* Create popup menu */
+	priv->menu = gtk_menu_new ();
+	item_rename = gtk_menu_item_new_with_label (_("Rename"));
+	item_remove = gtk_menu_item_new_with_label (_("Remove"));
+
+	g_signal_connect (item_rename, "activate", G_CALLBACK(on_rename), bookmarks);
+	g_signal_connect (item_remove, "activate", G_CALLBACK(on_remove_clicked), bookmarks);			
+
+	gtk_menu_shell_append (GTK_MENU_SHELL(priv->menu), item_rename);
+	gtk_menu_shell_append (GTK_MENU_SHELL(priv->menu), item_remove);
+
+	gtk_widget_show_all (priv->menu);
+	gtk_menu_attach_to_widget (GTK_MENU (priv->menu), priv->tree, NULL);
+	
 	gtk_widget_show_all (priv->window);
 }
 
@@ -250,6 +356,9 @@ anjuta_bookmarks_finalize (GObject *object)
 {
 	AnjutaBookmarks* bookmarks = ANJUTA_BOOKMARKS (object);
 	AnjutaBookmarksPrivate* priv = BOOKMARKS_GET_PRIVATE(bookmarks);
+
+	gtk_widget_destroy (priv->menu);
+	
 	anjuta_shell_remove_widget (ANJUTA_PLUGIN(priv->docman)->shell,
 								priv->window,
 								NULL);
@@ -275,14 +384,17 @@ anjuta_bookmarks_new (DocmanPlugin* docman)
 																 NULL));
 	AnjutaBookmarksPrivate* priv = BOOKMARKS_GET_PRIVATE(bookmarks);
 	priv->docman = docman;
+
 	
-	anjuta_shell_add_widget (ANJUTA_PLUGIN(docman)->shell,
-							 priv->window,
-							 "bookmarks",
-							 _("Bookmarks"),
-							 ANJUTA_STOCK_BOOKMARK_TOGGLE,
-							 ANJUTA_SHELL_PLACEMENT_RIGHT,
-							 NULL);
+	
+	anjuta_shell_add_widget_custom (ANJUTA_PLUGIN(docman)->shell,
+	                                priv->window,
+	                                "bookmarks",
+	                                _("Bookmarks"),
+	                                ANJUTA_STOCK_BOOKMARK_TOGGLE,
+	                                priv->grip,
+	                                ANJUTA_SHELL_PLACEMENT_RIGHT,
+	                                NULL);
 	
 	g_signal_connect (G_OBJECT(docman->docman), "document-changed",
 					  G_CALLBACK(on_document_changed), bookmarks);
@@ -363,7 +475,6 @@ anjuta_bookmarks_add (AnjutaBookmarks* bookmarks, IAnjutaEditor* editor, gint li
 	g_return_if_fail (IANJUTA_IS_MARKABLE(editor));
 	IAnjutaMarkable* markable = IANJUTA_MARKABLE(editor);
 	GtkTreeIter iter;
-	GtkTreePath* path;
 	gint handle;
 	gchar* text;
 	AnjutaBookmarksPrivate* priv = BOOKMARKS_GET_PRIVATE(bookmarks);
@@ -392,24 +503,6 @@ anjuta_bookmarks_add (AnjutaBookmarks* bookmarks, IAnjutaEditor* editor, gint li
 						-1);
 	g_free(text);
 	g_object_unref (file);
-
-	g_object_set (G_OBJECT(priv->renderer), "editable", TRUE, NULL);
-	
-	if (use_selection)
-	{
-		path = gtk_tree_model_get_path (priv->model, &iter);
-		anjuta_shell_present_widget (ANJUTA_PLUGIN(priv->docman)->shell,
-									 priv->window, NULL);
-		gtk_widget_grab_focus (priv->tree);
-		gtk_tree_view_scroll_to_cell (GTK_TREE_VIEW (priv->tree), path,
-									  priv->column, FALSE, 0.0, 0.0);
-		
-		gtk_tree_view_set_cursor_on_cell (GTK_TREE_VIEW (priv->tree), path,
-										  priv->column,
-										  priv->renderer,
-										  TRUE);
-		gtk_tree_path_free (path);
-	}
 }
 
 void
@@ -476,7 +569,7 @@ anjuta_bookmarks_add_file (AnjutaBookmarks* bookmarks,
 	
 	if ((doc = anjuta_docman_get_document_for_file (ANJUTA_DOCMAN(priv->docman->docman), file)))
 	{
-		anjuta_bookmarks_add (bookmarks, IANJUTA_EDITOR(doc), line, NULL, FALSE);
+		anjuta_bookmarks_add (bookmarks, IANJUTA_EDITOR(doc), line, title, FALSE);
 	}
 	else
 	{
