@@ -1,6 +1,6 @@
 /*
  * Copyright (C) 2002, 2003  Sebastian Rittau <srittau@jroger.in-berlin.de>
- * $Id: egg-datetime.c 4610 2009-01-19 17:17:01Z jhs $
+ * $Id$
  *
  * Based on GnomeDateEdit by Miguel de Icaza.
  *
@@ -535,7 +535,7 @@ egg_datetime_init (EggDateTime *edt)
 	cal_options = GTK_CALENDAR_SHOW_DAY_NAMES | GTK_CALENDAR_SHOW_HEADING;
 	if (priv->week_start_monday)
 		cal_options |= GTK_CALENDAR_WEEK_START_MONDAY;
-	gtk_calendar_display_options (GTK_CALENDAR (priv->calendar), cal_options);
+	gtk_calendar_set_display_options (GTK_CALENDAR (priv->calendar), cal_options);
 	gtk_box_pack_start(GTK_BOX(vbox), priv->calendar, TRUE, TRUE,0);
 	g_signal_connect_swapped (G_OBJECT (priv->calendar), "day-selected",
 			G_CALLBACK (cal_popup_changed), edt);
@@ -824,15 +824,17 @@ get_time_string (guint8 hour, guint8 minute, guint8 second)
 popup_position (GtkWidget *widget, GtkWindow *popup)
 {
 	GtkRequisition requisition;
+	GtkAllocation allocation;
 	gint x, y, width, height;
 
 	gtk_widget_size_request (GTK_WIDGET (popup), &requisition);
-	gdk_window_get_origin (widget->window, &x, &y);
+	gdk_window_get_origin (gtk_widget_get_window (widget), &x, &y);
 
-	x += widget->allocation.x;
-	y += widget->allocation.y;
-	width  = widget->allocation.width;
-	height = widget->allocation.height;
+	gtk_widget_get_allocation (widget, &allocation);
+	x += allocation.x;
+	y += allocation.y;
+	width  = allocation.width;
+	height = allocation.height;
 
 	x += width - requisition.width;
 	y += height;
@@ -932,7 +934,7 @@ cal_set_today (EggDateTime *edt, GtkCalendar *calendar)
 	edt->priv->no_date = FALSE;
 
 	now = g_date_new();
-	g_date_set_time (now, time (NULL));
+	g_date_set_time_t (now, time (NULL));
 
 	year = g_date_get_year(now);
 	month = g_date_get_month(now);
@@ -1029,7 +1031,7 @@ cal_popup_button_pressed (EggDateTime *edt, GdkEventButton *event, GtkWidget *wi
 		while (child) {
 			if (child == widget)
 				return FALSE;
-			child = child->parent;
+			child = gtk_widget_get_parent (child);
 		}
 	}
 
@@ -1103,7 +1105,8 @@ time_popup_changed (EggDateTime *edt, Timelist *timelist)
 
 	normalize_time (edt);
 
-	if(GTK_WIDGET_VISIBLE(timelist))update_time_label (edt);
+	if (gtk_widget_get_visible (timelist))
+		update_time_label (edt);
 
 	g_signal_emit (G_OBJECT (edt), egg_datetime_signals[SIGNAL_TIME_CHANGED], 0);
 }
@@ -1139,7 +1142,7 @@ time_popup_button_pressed (EggDateTime *edt, GdkEventButton *event, GtkWidget *w
 		while (child) {
 			if (child == widget)
 				return FALSE;
-			child = child->parent;
+			child = gtk_widget_get_parent (child);
 		}
 	}
 
