@@ -1,7 +1,7 @@
-/* -*- Mode: C; indent-tabs-mode: t; c-basic-offset: 8; tab-width: 8 -*- */
+/* -*- Mode: C; indent-tabs-mode: t; c-basic-offset: 4; tab-width: 4 -*- */
 /* gbf-tree-data.h
  *
- * Copyright (C) 2000  JP Rosevear
+ * Copyright (C) 2010  Sébastien Granjoux
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -17,49 +17,51 @@
  * License along with this program; if not, write to the
  * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
  * Boston, MA 02110-1301, USA.
- *
- * Author: JP Rosevear
  */
 
 #ifndef _GBF_TREE_DATA_H_
 #define _GBF_TREE_DATA_H_
 
 #include <glib-object.h>
-#include <libanjuta/gbf-project.h>
+#include <gtk/gtk.h>
+#include <libanjuta/interfaces/ianjuta-project.h>
+#include <libanjuta/anjuta-project.h>
 
 G_BEGIN_DECLS
 
-#define GBF_TYPE_TREE_DATA			(gbf_tree_data_get_type ())
-
 typedef struct _GbfTreeData GbfTreeData;
 
 typedef enum {
 	GBF_TREE_NODE_STRING,
 	GBF_TREE_NODE_GROUP,
 	GBF_TREE_NODE_TARGET,
-	GBF_TREE_NODE_TARGET_SOURCE,
+	GBF_TREE_NODE_SOURCE,
+	GBF_TREE_NODE_SHORTCUT
 } GbfTreeNodeType;
 
 struct _GbfTreeData
 {
-	GbfTreeNodeType  type;
+	GbfTreeNodeType  	type;
 	gchar           *name;
-	gchar           *id;
-	gchar           *uri;
-	gboolean         is_shortcut;
-	gchar           *mime_type;
+	GFile			*group;
+	gchar			*target;
+	GFile			*source;
+	gboolean		is_shortcut;
+	GbfTreeData		*shortcut;
+	GtkWidget		*properties_dialog;
 };
 
-GType          gbf_tree_data_get_type            (void);
-GbfTreeData   *gbf_tree_data_new_string          (const gchar                  *string);
-GbfTreeData   *gbf_tree_data_new_group           (GbfProject                   *project,
-						  const GbfProjectGroup        *group);
-GbfTreeData   *gbf_tree_data_new_target          (GbfProject                   *project,
-						  const GbfProjectTarget       *target);
-GbfTreeData   *gbf_tree_data_new_source          (GbfProject                   *project,
-						  const GbfProjectTargetSource *source);
-GbfTreeData   *gbf_tree_data_copy                (GbfTreeData                  *data);
-void           gbf_tree_data_free                (GbfTreeData                  *data);
+AnjutaProjectNode *gbf_tree_data_get_node	 (GbfTreeData		 *data,
+                                              IAnjutaProject     *project);
+
+gchar	      *gbf_tree_data_get_uri		 (GbfTreeData            *data);
+
+GbfTreeData   *gbf_tree_data_new_string      (const gchar          *string);
+GbfTreeData   *gbf_tree_data_new_shortcut    (GbfTreeData		   *src);
+GbfTreeData   *gbf_tree_data_new_group       (AnjutaProjectGroup   *group);
+GbfTreeData   *gbf_tree_data_new_target      (AnjutaProjectTarget  *target);
+GbfTreeData   *gbf_tree_data_new_source      (AnjutaProjectSource  *source);
+void           gbf_tree_data_free            (GbfTreeData          *data);
 
 
 G_END_DECLS
